// Code generated by smithy-go-codegen DO NOT EDIT.

package accessanalyzer

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/accessanalyzer/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Checks whether a resource policy can grant public access to the specified
// resource type.
func (c *Client) CheckNoPublicAccess(ctx context.Context, params *CheckNoPublicAccessInput, optFns ...func(*Options)) (*CheckNoPublicAccessOutput, error) {
	if params == nil {
		params = &CheckNoPublicAccessInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "CheckNoPublicAccess", params, optFns, c.addOperationCheckNoPublicAccessMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*CheckNoPublicAccessOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type CheckNoPublicAccessInput struct {

	// The JSON policy document to evaluate for public access.
	//
	// This member is required.
	PolicyDocument *string

	// The type of resource to evaluate for public access. For example, to check for
	// public access to Amazon S3 buckets, you can choose AWS::S3::Bucket for the
	// resource type.
	//
	// For resource types not supported as valid values, IAM Access Analyzer will
	// return an error.
	//
	// This member is required.
	ResourceType types.AccessCheckResourceType

	noSmithyDocumentSerde
}

type CheckNoPublicAccessOutput struct {

	// The message indicating whether the specified policy allows public access to
	// resources.
	Message *string

	// A list of reasons why the specified resource policy grants public access for
	// the resource type.
	Reasons []types.ReasonSummary

	// The result of the check for public access to the specified resource type. If
	// the result is PASS , the policy doesn't allow public access to the specified
	// resource type. If the result is FAIL , the policy might allow public access to
	// the specified resource type.
	Result types.CheckNoPublicAccessResult

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationCheckNoPublicAccessMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpCheckNoPublicAccess{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpCheckNoPublicAccess{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "CheckNoPublicAccess"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addTimeOffsetBuild(stack, c); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = addOpCheckNoPublicAccessValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opCheckNoPublicAccess(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opCheckNoPublicAccess(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "CheckNoPublicAccess",
	}
}
