// Code generated by smithy-go-codegen DO NOT EDIT.

package comprehend

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/comprehend/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Gets a list of the properties of all entity recognizers that you created,
// including recognizers currently in training. Allows you to filter the list of
// recognizers based on criteria such as status and submission time. This call
// returns up to 500 entity recognizers in the list, with a default number of 100
// recognizers in the list.
//
// The results of this list are not in any particular order. Please get the list
// and sort locally if needed.
func (c *Client) ListEntityRecognizers(ctx context.Context, params *ListEntityRecognizersInput, optFns ...func(*Options)) (*ListEntityRecognizersOutput, error) {
	if params == nil {
		params = &ListEntityRecognizersInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListEntityRecognizers", params, optFns, c.addOperationListEntityRecognizersMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListEntityRecognizersOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListEntityRecognizersInput struct {

	// Filters the list of entities returned. You can filter on Status ,
	// SubmitTimeBefore , or SubmitTimeAfter . You can only set one filter at a time.
	Filter *types.EntityRecognizerFilter

	//  The maximum number of results to return on each page. The default is 100.
	MaxResults *int32

	// Identifies the next page of results to return.
	NextToken *string

	noSmithyDocumentSerde
}

type ListEntityRecognizersOutput struct {

	// The list of properties of an entity recognizer.
	EntityRecognizerPropertiesList []types.EntityRecognizerProperties

	// Identifies the next page of results to return.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListEntityRecognizersMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpListEntityRecognizers{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpListEntityRecognizers{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "ListEntityRecognizers"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addTimeOffsetBuild(stack, c); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListEntityRecognizers(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

// ListEntityRecognizersPaginatorOptions is the paginator options for
// ListEntityRecognizers
type ListEntityRecognizersPaginatorOptions struct {
	//  The maximum number of results to return on each page. The default is 100.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListEntityRecognizersPaginator is a paginator for ListEntityRecognizers
type ListEntityRecognizersPaginator struct {
	options   ListEntityRecognizersPaginatorOptions
	client    ListEntityRecognizersAPIClient
	params    *ListEntityRecognizersInput
	nextToken *string
	firstPage bool
}

// NewListEntityRecognizersPaginator returns a new ListEntityRecognizersPaginator
func NewListEntityRecognizersPaginator(client ListEntityRecognizersAPIClient, params *ListEntityRecognizersInput, optFns ...func(*ListEntityRecognizersPaginatorOptions)) *ListEntityRecognizersPaginator {
	if params == nil {
		params = &ListEntityRecognizersInput{}
	}

	options := ListEntityRecognizersPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListEntityRecognizersPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListEntityRecognizersPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListEntityRecognizers page.
func (p *ListEntityRecognizersPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListEntityRecognizersOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	optFns = append([]func(*Options){
		addIsPaginatorUserAgent,
	}, optFns...)
	result, err := p.client.ListEntityRecognizers(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

// ListEntityRecognizersAPIClient is a client that implements the
// ListEntityRecognizers operation.
type ListEntityRecognizersAPIClient interface {
	ListEntityRecognizers(context.Context, *ListEntityRecognizersInput, ...func(*Options)) (*ListEntityRecognizersOutput, error)
}

var _ ListEntityRecognizersAPIClient = (*Client)(nil)

func newServiceMetadataMiddleware_opListEntityRecognizers(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "ListEntityRecognizers",
	}
}
