// Code generated by smithy-go-codegen DO NOT EDIT.

package configservice

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/configservice/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Returns the details of one or more retention configurations. If the retention
// configuration name is not specified, this action returns the details for all the
// retention configurations for that account.
//
// Currently, Config supports only one retention configuration per region in your
// account.
func (c *Client) DescribeRetentionConfigurations(ctx context.Context, params *DescribeRetentionConfigurationsInput, optFns ...func(*Options)) (*DescribeRetentionConfigurationsOutput, error) {
	if params == nil {
		params = &DescribeRetentionConfigurationsInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "DescribeRetentionConfigurations", params, optFns, c.addOperationDescribeRetentionConfigurationsMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*DescribeRetentionConfigurationsOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type DescribeRetentionConfigurationsInput struct {

	// The nextToken string returned on a previous page that you use to get the next
	// page of results in a paginated response.
	NextToken *string

	// A list of names of retention configurations for which you want details. If you
	// do not specify a name, Config returns details for all the retention
	// configurations for that account.
	//
	// Currently, Config supports only one retention configuration per region in your
	// account.
	RetentionConfigurationNames []string

	noSmithyDocumentSerde
}

type DescribeRetentionConfigurationsOutput struct {

	// The nextToken string returned on a previous page that you use to get the next
	// page of results in a paginated response.
	NextToken *string

	// Returns a retention configuration object.
	RetentionConfigurations []types.RetentionConfiguration

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationDescribeRetentionConfigurationsMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpDescribeRetentionConfigurations{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpDescribeRetentionConfigurations{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "DescribeRetentionConfigurations"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addTimeOffsetBuild(stack, c); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opDescribeRetentionConfigurations(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

// DescribeRetentionConfigurationsPaginatorOptions is the paginator options for
// DescribeRetentionConfigurations
type DescribeRetentionConfigurationsPaginatorOptions struct {
	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// DescribeRetentionConfigurationsPaginator is a paginator for
// DescribeRetentionConfigurations
type DescribeRetentionConfigurationsPaginator struct {
	options   DescribeRetentionConfigurationsPaginatorOptions
	client    DescribeRetentionConfigurationsAPIClient
	params    *DescribeRetentionConfigurationsInput
	nextToken *string
	firstPage bool
}

// NewDescribeRetentionConfigurationsPaginator returns a new
// DescribeRetentionConfigurationsPaginator
func NewDescribeRetentionConfigurationsPaginator(client DescribeRetentionConfigurationsAPIClient, params *DescribeRetentionConfigurationsInput, optFns ...func(*DescribeRetentionConfigurationsPaginatorOptions)) *DescribeRetentionConfigurationsPaginator {
	if params == nil {
		params = &DescribeRetentionConfigurationsInput{}
	}

	options := DescribeRetentionConfigurationsPaginatorOptions{}

	for _, fn := range optFns {
		fn(&options)
	}

	return &DescribeRetentionConfigurationsPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *DescribeRetentionConfigurationsPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next DescribeRetentionConfigurations page.
func (p *DescribeRetentionConfigurationsPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*DescribeRetentionConfigurationsOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	optFns = append([]func(*Options){
		addIsPaginatorUserAgent,
	}, optFns...)
	result, err := p.client.DescribeRetentionConfigurations(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

// DescribeRetentionConfigurationsAPIClient is a client that implements the
// DescribeRetentionConfigurations operation.
type DescribeRetentionConfigurationsAPIClient interface {
	DescribeRetentionConfigurations(context.Context, *DescribeRetentionConfigurationsInput, ...func(*Options)) (*DescribeRetentionConfigurationsOutput, error)
}

var _ DescribeRetentionConfigurationsAPIClient = (*Client)(nil)

func newServiceMetadataMiddleware_opDescribeRetentionConfigurations(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "DescribeRetentionConfigurations",
	}
}
